from dataclass_factory import Factory, Schema
from waio.factory.models.basic import PayloadBaseModel
from waio.factory.models.button import PayloadButtonReply
from waio.factory.models.document import PayloadFile
from waio.factory.models.image import PayloadImage
from waio.factory.models.list import PayloadList
from waio.factory.models.text import PayloadText
from waio.factory.schemas import Schemas
from waio.factory.models.response import ResponseMessagePayload, ResponseMessagePayloadPayload, ResponseMessage
from waio.factory.models.contact import (
    PayloadContact,
    PayloadContactAddress,
    PayloadContactEmail,
    PayloadContactInformation,
    PayloadContactPhone,
    PayloadContactOrganization,
    PayloadContacts,
    PayloadContactUrl,
    PayloadSender
)


factory_gupshup = Factory(
    schemas={
        PayloadBaseModel: Schemas.BASE_MODEL,
        PayloadText: Schemas.PAYLOAD_TEXT,
        PayloadList: Schemas.PAYLOAD_LIST,
        PayloadSender: Schemas.PAYLOAD_SENDER,
        PayloadButtonReply: Schemas.PAYLOAD_REPLY_KEYBOARD,
        PayloadImage: Schemas.PAYLOAD_IMAGE,
        PayloadFile: Schemas.PAYLOAD_FILE,
        PayloadContact: Schemas.PAYLOAD_CONTACT,
        PayloadContacts: Schemas.CONTACTS_PAYLOAD,
        PayloadContactOrganization: Schemas.PAYLOAD_CONTACT_ORGANIZATION,
        PayloadContactPhone: Schemas.PAYLOAD_CONTACT_PHONE,
        PayloadContactUrl: Schemas.PAYLOAD_CONTACT_URL,
        PayloadContactInformation: Schemas.PAYLOAD_CONTACT_INFORMATION,
        PayloadContactEmail: Schemas.PAYLOAD_CONTACT_EMAIL,
        PayloadContactAddress: Schemas.PAYLOAD_CONTACT_ADDRESS,
        ResponseMessage: Schemas.RESPONSE_MESSAGE,
        ResponseMessagePayload: Schemas.RESPONSE_MESSAGE_PAYLOAD,
        ResponseMessagePayloadPayload: Schemas.RESPONSE_MESSAGE_PAYLOAD_PAYLOAD
}, debug_path=True)

factory_default = Factory(
    default_schema=Schema(omit_default=True),
    debug_path=True
)



from dataclasses import dataclass
from typing import List, Optional, Union


@dataclass
class PayloadSender:
    phone: int
    name: str


@dataclass
class BaseModel:
    sender: PayloadSender
    payload_id: str


@dataclass
class PayloadContactName:
    first_name: str
    formatted_name: str
    last_name: Optional[str] = None


@dataclass
class PayloadContactPhone:
    phone: Optional[str] = None
    type: Optional[str] = None


@dataclass
class PayloadContactAddress:
    city: Optional[str] = None
    country: Optional[str] = None
    country_code: Optional[str] = None
    state: Optional[str] = None
    street: Optional[str] = None
    type: Optional[str] = None
    zip: Optional[str] = None


@dataclass
class PayloadContactEmail:
    email: Optional[str] = None
    type: Optional[str] = None


@dataclass
class PayloadContactOrganization:
    company: Union[str] = None


@dataclass
class PayloadContactUrl:
    url: Optional[str] = None
    type: Optional[str] = None


@dataclass
class PayloadContactInformation:
    service: Optional[str] = None
    user_id: Optional[str] = None


@dataclass
class PayloadContact:
    emails: Optional[List[PayloadContactEmail]] = None
    ims: Optional[List[PayloadContactInformation]] = None
    name: Optional[PayloadContactName] = None
    org: Optional[PayloadContactOrganization] = None
    phones: Optional[List[PayloadContactPhone]] = None
    urls: Optional[List[PayloadContactUrl]] = None
    addresses: Optional[List[PayloadContactAddress]] = None


@dataclass
class PayloadContacts(BaseModel):
    contacts: List[PayloadContact]



import inspect
from typing import Optional, Dict, Any

from waio.handlers import ABCHandler
from waio.types.message import Message


class HandlerExecutor:

    @classmethod
    async def execute(cls, handler: ABCHandler, message: Message, **middleware_kwargs) -> Optional[Dict[str, Any]]:
        handler_filter = await handler.filter(message)

        if isinstance(handler_filter, dict):
            values_handler = {"message": message, "state": message.state, **handler_filter, **middleware_kwargs}
            func_info = inspect.getfullargspec(handler.handler)
            values_to_func = {key: value for key, value in values_handler.items() if key in func_info.args}
            return values_to_func
        return None



from typing import Protocol
from waio.keyboard.list import ListMessage
from waio.keyboard.reply import QuickReply


class Bot(Protocol):

    async def send_message(self, receiver: int, message: str):
        """Send message"""

    async def send_list(self, receiver: int, keyboard: ListMessage):
        """Send list message"""

    async def send_reply(self, receiver: int, keyboard: QuickReply):
        """Send reply message"""



from dataclasses import dataclass
from typing import Union

from waio.factory.models.button import PayloadButtonReply
from waio.factory.models.contact import PayloadContacts
from waio.factory.models.document import PayloadFile
from waio.factory.models.image import PayloadImage
from waio.factory.models.list import PayloadList
from waio.factory.models.text import PayloadText


@dataclass
class BaseResponse:
    app: str
    type: str
    payload: Union[
        PayloadList,
        PayloadText,
        PayloadButtonReply,
        PayloadImage,
        PayloadFile,
        PayloadContacts
    ]



from abc import ABC, abstractmethod
from typing import Dict, Any

from waio.types import Message
from waio.utils.callback.base_callback import CallbackDataBase


class CallbackDataFilterBase(ABC):

    def __init__(self, factory: CallbackDataBase, config: Dict[str, str]):
        self.config = config
        self.factory = factory

    @classmethod
    def validate(cls, full_config: Dict[str, Any]):
        raise ValueError("That filter can't be used in filters factory!")

    async def base_check(self, data):
        try:
            data = self.factory.parse(data)
        except ValueError:
            return False
        for key, value in self.config.items():
            if isinstance(value, (list, tuple, set, frozenset)):
                if data.get(key) not in value:
                    return False
            elif data.get(key) != value:
                return False
        return {'callback_data': data}

    @abstractmethod
    async def check(self, message: Message):
        pass



from abc import ABC, abstractmethod
from typing import Any, Union, Callable

from waio.rules.abc import ABCRule
from waio.types import Message


class ABCHandler(ABC):

    @abstractmethod
    def __init__(self, handler: Callable, *rules: ABCRule):
        self.handler = handler
        self.rules = rules

    @abstractmethod
    async def filter(self, event: "Message") -> Union[dict, bool]:
        pass

    @abstractmethod
    async def handle(self, event: "Message", **context) -> Any:
        pass



from dataclasses import dataclass
from typing import Optional

from waio.factory.models.basic import PayloadBaseModel


@dataclass
class PayloadList(PayloadBaseModel):
    title: Optional[str]
    id: Optional[str] = None
    reply: Optional[str] = None
    postback_text: Optional[str] = None
    description: Optional[str] = None



from dataclass_factory import Schema
from dataclass_factory.schema_helpers import type_checker


class Schemas:
    PAYLOAD_TEXT = Schema(
        name_mapping={
            "payload_id": "id",
            "type": "type",
            "text": ("payload", "text"),
        },
        skip_internal=True
    )

    PAYLOAD_SENDER = Schema(
        name_mapping={
            "phone": "phone",
            "name": "name",
        },
        skip_internal=True
    )

    PAYLOAD_LIST = Schema(
        name_mapping={
            "payload_id": "id",
            "type": "type",
            "title": ("payload", "title"),
            "id": ("payload", "id"),
            "reply": ("payload", "reply"),
            "postback_text": ("payload", "postbackText"),
            "description": ("payload", "description"),
        },
        pre_parse=type_checker("list_reply", field="type"),
        skip_internal=True
    )

    PAYLOAD_REPLY_KEYBOARD = Schema(
        name_mapping={
            "payload_id": "id",
            "type": "type",
            "title": ("payload", "title"),
            "id": ("payload", "id"),
            "reply": ("payload", "reply"),
        },
        pre_parse=type_checker(value="button_reply", field="type"),
        skip_internal=True
    )

    PAYLOAD_IMAGE = Schema(
        name_mapping={
            "payload_id": "id",
            "type": "type",
            "url": ("payload", "url"),
            "content_type": ("payload", "contentType"),
            "url_expiry": ("payload", "urlExpiry"),
            "caption": ("payload", "caption")
        },
        pre_parse=type_checker(value="image", field="type"),
        skip_internal=True
    )

    PAYLOAD_FILE = Schema(
        name_mapping={
            "payload_id": "id",
            "type": "type",
            "url": ("payload", "url"),
            "content_type": ("payload", "contentType"),
            "url_expiry": ("payload", "urlExpiry"),
            "caption": ("payload", "caption"),
            "name": ("payload", "name"),
        },
        pre_parse=type_checker(value="file", field="type"),
        skip_internal=True
    )

    BASE_MODEL = Schema(
        name_mapping={
            "payload_id": "id",
            "type": "type",
        },
        skip_internal=True
    )

    CONTACTS_PAYLOAD = Schema(
        name_mapping={
            "payload_id": "id",
            "contacts": ("payload", "contacts"),
        },
        skip_internal=True
    )

    PAYLOAD_CONTACT_PHONE = Schema(
        name_mapping={
            "phone": "phone",
            "type": "type",
        },
        skip_internal=True
    )

    PAYLOAD_CONTACT_ORGANIZATION = Schema(
        name_mapping={
            "company": "company",
        },
        skip_internal=True
    )

    PAYLOAD_CONTACT_URL = Schema(
        name_mapping={
            "url": "url",
            "type": "type"
        },
        skip_internal=True
    )

    PAYLOAD_CONTACT_INFORMATION = Schema(
        name_mapping={
            "service": "service",
            "user_id": "user_id"
        },
        skip_internal=True
    )

    PAYLOAD_CONTACT = Schema(
        name_mapping={
            "addresses": "addresses",
            "emails": "emails",
            "ims": "ims",
            "name": "name",
            "org": "org",
            "phones": "phones",
            "urls": "urls",
        },
        skip_internal=True
    )

    PAYLOAD_CONTACT_EMAIL = Schema(
        name_mapping={
            "email": "email",
            "type": "type"
        },
        skip_internal=True
    )
    PAYLOAD_CONTACT_ADDRESS = Schema(
        name_mapping={
            "city": "city",
            "country": "country",
            "country_code": "countryCode",
            "state": "state",
            "street": "street",
            "type": "type",
            "zip": "zip"
        },
        skip_internal=True
    )

    RESPONSE_MESSAGE_PAYLOAD_PAYLOAD = Schema(
        name_mapping={
            "whatsapp_Message_id": "whatsappMessageId",
            "type": "type",
        },
        skip_internal=True
    )

    RESPONSE_MESSAGE_PAYLOAD = Schema(
        name_mapping={
            "id": "id",
            "type": "type",
            "destination": "destination",
            "payload": "payload"
        },
        skip_internal=True
    )

    RESPONSE_MESSAGE = Schema(
        name_mapping={
            "app": "app",
            "timestamp": "timestamp",
            "version": "version",
            "type": "type",
            "payload": "payload"
        },
        skip_internal=True
    )



from waio.types import Message
from waio.utils.callback.base_filter import CallbackDataFilterBase


class CallbackDataFilterGroup(CallbackDataFilterBase):
    async def check(self, message: Message):
        return await self.base_check(message.callback_data_list)


class CallbackDataFilterItem(CallbackDataFilterBase):
    async def check(self, message: Message):
        return await self.base_check(message.callback_data_item)



from abc import ABC
from typing import List, Any, Dict, Optional

from waio.handlers import ABCHandler
from waio.types import Message


class BaseMiddleware(ABC):

    event: Message
    handle_responses: Optional[List[Dict[str, Any]]]
    handlers: Optional[List[ABCHandler]]

    def fill(
            self,
            event: Message,
            handle_responses: Optional[List[Dict[str, Any]]] = None,
            handlers: Optional[List[ABCHandler]] = None,
    ):
        self.event = event
        self.handle_responses = handle_responses
        self.handlers = handlers

    async def pre(self):
        pass

    async def post(self):
        pass

    def __repr__(self) -> str:
        return (f"<{self.__class__.__name__} "
                f"event: {self.event}, "
                f"handlers: {self.handlers}, "
                f"returns: {self.handle_responses}>")



from typing import Callable, Any

from waio.factory.base import ResponseModel
from waio.protocols.bot import Bot
from waio.states.context import FSMContext


class MessagePropertyEvent:
    def __init__(self, message: ResponseModel):
        self.message = message

    @property
    def text(self) -> str:
        if hasattr(self.message.payload, 'caption') and self.message.payload.caption:
            return self.message.payload.caption
        if hasattr(self.message.payload, 'text') and self.message.payload.text:
            return self.message.payload.text
        return ''

    @property
    def sender_number(self):
        return self.message.payload.sender.phone

    @property
    def sender_name(self):
        return self.message.payload.sender.name

    @property
    def callback_data_item(self):
        if hasattr(self.message.payload, 'postback_text') and self.message.payload.postback_text:
            return self.message.payload.postback_text
        return ''

    @property
    def callback_data_list(self):
        if hasattr(self.message.payload, 'id') and self.message.payload.id:
            return self.message.payload.id
        return ''


class Message(MessagePropertyEvent):
    def __init__(self, bot: Bot, message: ResponseModel, state_func: Callable[[int], FSMContext]):
        self.bot = bot
        self.message = message
        self._state_func = state_func

        super().__init__(message)

    @property
    def state(self):
        return self._state_func(self.message.payload.sender.phone)

    async def current_state(self) -> str:
        current_state = await self.state.get_state()
        return str(current_state)

    async def answer(self, message: str):
        await self.bot.send_message(receiver=self.message.payload.sender.phone, message=message)



from enum import Enum

from waio.factory.models.button import PayloadButtonReply
from waio.factory.models.document import PayloadFile
from waio.factory.models.image import PayloadImage
from waio.factory.models.list import PayloadList
from waio.factory.models.text import PayloadText
from waio.factory.models.contact import PayloadContacts


class ContentType(Enum):
    TEXT = PayloadText
    FILE = PayloadFile
    PHOTO = PayloadImage
    CONTACT = PayloadContacts
    BUTTON = PayloadButtonReply
    LIST = PayloadList




from aiohttp import FormData
from typing import Any, Optional, Dict


class CustomFormDataStorage(FormData):
    def add_www_form(self, name: str, value: Any):
        self.add_field(name=name, value=value)

    def add_multipart_form(self, name: str, filename: Optional[str], value: Any, content_type: Optional[str] = None):
        self.add_field(name=name, filename=filename, value=value, content_type=content_type)


class CustomFormData(CustomFormDataStorage):
    async def upload(self, key, value: str):
        if isinstance(value, str):
            self.add_www_form(name=key, value=value)

    async def uploads(self, elements: Dict[str, Any]):
        if isinstance(elements, dict):
            for key, value in elements.items():
                await self.upload(key=key, value=value)

    def __iter__(self):
        data = self.__dict__.get('_fields')
        for element in data:
            yield element[0]['name'], element[2]



from waio.utils.callback.base_callback import CallbackDataBase
from waio.utils.callback.filters import CallbackDataFilterItem, CallbackDataFilterGroup


class CallbackDataItem(CallbackDataBase):

    def filter(self, **config) -> 'CallbackDataFilterItem':
        for key in config.keys():
            if key not in self._part_names:
                raise ValueError(f'Invalid field name {key!r}')
        return CallbackDataFilterItem(self, config)


class CallbackDataGroup(CallbackDataBase):

    def filter(self, **config) -> 'CallbackDataFilterGroup':
        for key in config.keys():
            if key not in self._part_names:
                raise ValueError(f'Invalid field name {key!r}')
        return CallbackDataFilterGroup(self, config)


from abc import abstractmethod, ABC
from typing import Dict


class CallbackDataBase(ABC):
    """
    Callback data factory
    """

    def __init__(self, prefix, *parts, sep=':'):
        if not isinstance(prefix, str):
            raise TypeError(f'Prefix must be instance of str not {type(prefix).__name__}')
        if not prefix:
            raise ValueError("Prefix can't be empty")
        if sep in prefix:
            raise ValueError(f"Separator {sep!r} can't be used in prefix")

        self.prefix = prefix
        self.sep = sep

        self._part_names = parts

    def new(self, *args, **kwargs) -> str:
        """
        Generate callback data
        :param args:
        :param kwargs:
        :return:
        """
        args = list(args)

        data = [self.prefix]

        for part in self._part_names:
            value = kwargs.pop(part, None)
            if value is None:
                if args:
                    value = args.pop(0)
                else:
                    raise ValueError(f'Value for {part!r} was not passed!')

            if value is not None and not isinstance(value, str):
                value = str(value)

            if self.sep in value:
                raise ValueError(f"Symbol {self.sep!r} is defined as the separator and can't be used in parts' values")

            data.append(value)

        if args or kwargs:
            raise TypeError('Too many arguments were passed!')

        callback_data = self.sep.join(data)
        if len(callback_data.encode()) > 64:
            raise ValueError('Resulted callback data is too long!')

        return callback_data

    def parse(self, callback_data: str) -> Dict[str, str]:
        """
        Parse data from the callback data
        :param callback_data:
        :return:
        """
        prefix, *parts = callback_data.split(self.sep)
        if prefix != self.prefix:
            raise ValueError("Passed callback data can't be parsed with that prefix.")
        elif len(parts) != len(self._part_names):
            raise ValueError('Invalid parts count!')

        result = {'@': prefix}
        result.update(zip(self._part_names, parts))
        return result

    @abstractmethod
    def filter(self, **config):
        pass



from typing import Optional, TypeVar
import aioredis

T = TypeVar("T", bound="RedisConnector")


class RedisConnector:
    def __init__(self, redis_url: str, db_number: int = 1, encoding: Optional[str] = "utf-8"):
        self._redis: Optional[aioredis.Redis] = None
        self.redis_url = redis_url
        self.db = db_number
        self.encoding = encoding

    @property
    def redis(self) -> aioredis.Redis:
        if not self.check_connection:
            self.create_session()
        return self._redis

    @property
    def check_connection(self) -> bool:
        conn_attr = hasattr(self._redis, 'connection')
        if conn_attr:
            if self._redis.connection:
                return True
        return False

    def create_session(self) -> None:
        self._redis = aioredis.from_url(url=self.redis_url, encoding=self.encoding, db=self.db, decode_responses=True)

    async def close_session(self) -> None:
        await self._redis.close()



from typing import Optional, Union, List
import ujson

from waio.utils.dicts import clear_none_values


class ListMainButton:
    def __init__(self, title: str):
        self.type = 'text'
        self.title = title

    def json(self):
        return {"type": self.type, "title": self.title}


class ListGroupItem:
    def __init__(
            self,
            title: str,
            callback_data: Optional[str] = None,
            description: Optional[str] = None,
    ):
        self.type = 'text'
        self.title = title  # Maximum length: 24 characters
        self.description = description  # Maximum length: 72 characters
        self.callback_data = callback_data

    def json(self):
        return {
            "type": self.type,
            "title": self.title,
            "description": self.description,
            "postbackText": self.callback_data
        }


class ListGroup:
    def __init__(self, title: str, subtitle: str):
        self.title = title
        self.subtitle = subtitle
        self.options: List[ListGroupItem] = []

    def add(self, list_item: ListGroupItem) -> 'ListGroup':
        self.options.append(list_item)
        return self

    def json(self):

        d = {
            "title": self.title,
            "subtitle": self.subtitle,
            "options": [option.json() for option in self.options],
        }
        return clear_none_values(d)


class ListMessage:
    def __init__(
            self,
            title: str,
            body: str,
            button_title: str,
            items: Optional[List[ListGroup]] = None,
            callback_data: Optional[Union[int, str]] = None,
    ):
        self.type = 'list'
        self.title = title
        self.body = body
        self.button_title = button_title
        self.callback_data = callback_data

        if items is None:
            self.items = []
        else:
            self.items = items

    @property
    def main_button(self):
        return ListMainButton(title=self.button_title)

    def json(self):
        d = {
            "type": self.type,
            "title": self.title,
            "body": self.body,
            "msgid": self.callback_data,
            "globalButtons": [self.main_button.json()],
            "items": [item.json() for item in self.items]
        }
        return ujson.dumps(d, indent=2)



from dataclasses import dataclass
from typing import Optional

from waio.factory.models.image import PayloadImage


@dataclass
class PayloadFile(PayloadImage):
    caption: Optional[str] = None
    name: Optional[str] = None



from loguru import logger


class LogFilterLevel:

    def __init__(self, lvl):
        self.level = lvl

    def __call__(self, record):
        level_no = logger.level(self.level).no
        return record["level"].no >= level_no

    def set_level(self, lvl):
        self.level = lvl


loguru_filter = LogFilterLevel('WARNING')




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['waio',
 'waio.client',
 'waio.factory',
 'waio.factory.models',
 'waio.gupshup',
 'waio.handlers',
 'waio.keyboard',
 'waio.logs',
 'waio.middleware',
 'waio.models',
 'waio.protocols',
 'waio.rules',
 'waio.states',
 'waio.storage',
 'waio.types',
 'waio.utils',
 'waio.utils.callback']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.4.post0,<4.0.0',
 'aioredis==2.0.0',
 'dataclass-factory>=2.11,<3.0',
 'loguru>=0.5.3,<0.6.0',
 'ujson>=4.2.0,<5.0.0']

setup_kwargs = {
    'name': 'waio',
    'version': '0.1.1',
    'description': 'waio - is a pretty simple and fully asynchronous framework for WhatsApp Business API written in Python 3.7 with asyncio and aiohttp. Used API official Facebook partner - Gupshup.io',
    'long_description': '<p align="center">\n<img alt="Codacy grade" src="https://img.shields.io/codacy/grade/36695d0fb1c648fbb8e0cb00f3612c7e">\n<img src="https://scrutinizer-ci.com/g/dotX12/waio/badges/quality-score.png?b=master" alt="https://scrutinizer-ci.com/g/dotX12/waio/">\n<img src="https://scrutinizer-ci.com/g/dotX12/waio/badges/code-intelligence.svg?b=master" alt="https://scrutinizer-ci.com/g/dotX12/waio/">\n<img src="https://img.shields.io/testspace/tests/dotX12/66672/master" alt="Testspace tests">\n<img src="https://scrutinizer-ci.com/g/dotX12/waio/badges/build.png?b=master" alt="https://scrutinizer-ci.com/g/dotX12/waio/">\n<img src="https://badge.fury.io/py/waio.svg" alt="https://badge.fury.io/py/waio">\n<img src="https://img.shields.io/github/license/dotX12/waio.svg" alt="https://github.com/dotX12/waio/blob/master/LICENSE.md">\n<img src="https://img.shields.io/github/last-commit/dotX12/waio" alt="GitHub last commit">\n<br><br>\n<img width="1000" src="https://raw.githubusercontent.com/dotX12/waio/master/docs/assets/images/readme_footer.png">\n 🤖 waio - is a pretty simple and fully asynchronous framework for WhatsApp Business API written in Python 3.7 with asyncio and aiohttp.\nUsed API official Facebook partner - <a href="https://gupshup.io">Gupshup.io</a>\n\n-----\n</p>\n\n\nYou can create an account and use a **Sandbox** to develop a bot\nand test functionality, and when you finish - create your own business\nnumber and transfer the functionality to your number,\nmake verification (check mark) and be cool!\n',
    'author': 'dotX12',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/dotX12/waio',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
